#
#   5B - Controls
#

from albow.controls import TextField


class TextAttrField(TextField):

	def __init__(self, width, client, attr = 'value', upper = False, **kwds):
		TextField.__init__(self, width = width, **kwds)
		self.client = client
		self.attr = attr
		self.upper = upper
		self.editing = False
	
	def get_text(self):
		if self.editing:
			return TextField.get_text(self)
		else:
			x = getattr(self.client, self.attr)
			return self.value_to_text(x)

	def set_text(self, x):
		if self.editing:
			TextField.set_text(self, x)
		else:
			setattr(self.client, self.attr, self.text_to_value(x))
	
	def value_to_text(self, x):
		return x
	
	def text_to_value(self, x):
		return x

	def insert_char(self, c):
		if self.upper:
			c = c.upper()
		if not self.editing:
			text = self.get_text()
			self.editing = True
			self.set_text(text)
		return TextField.insert_char(self, c)
	
	def enter_action(self):
		if self.editing:
			text = self.get_text()
			self.editing = False
			self.set_text(text)
			self.parent.focus_on(None)


class TimeAttrField(TextAttrField):

	def value_to_text(self, secs):
		return "%d:%02d" % (secs // 60, secs % 60)
	
	def text_to_value(self, x):
		t = "".join([c for c in x if c in ":0123456789"])
		if ":" not in t:
			t = ":" + t
		m, s = t.split(":")[:2]
		def i(v):
			return int(v or "0")
		return i(m) * 60 + i(s)
