#
#   5B - Central game state
#

import os
from tempfile import mkstemp
from glob import glob
from level import Level
from reader import Reader
import settings

class Game(object):
	#  current_level          Level
	
	def __init__(self):
		#  ALL instance variables must be initialised here and not as
		#  class variables, so that game load/save works correctly.
		self.current_level = None
		self.current_level_dir = None

	def start_enabled(self):
		return not self.level_running()
		
	def resume_enabled(self):
		return self.level_running()

	def abort_enabled(self):
		return self.level_running()

	def level_present(self):
		return self.current_level is not None
	
	def level_running(self):
		level = self.current_level
		if level:
			return level.playing()
		else:
			return False
	
	def start_level(self):
		self.current_level.restart()
	
	def abort_level(self):
		level = self.current_level
		if level:
			level.reset()

	def create_level(self):
		self.current_level = Level()

	def load_level(self, path):
		f = open(path, "rU")
		try:
			r = Reader(f, path)
			self.current_level = Level.create_from_file(r)
		finally:
			f.close()

	def save_level(self, path):
		level = self.current_level
		fd, temp_path = mkstemp(dir = os.path.dirname(path), text = True)
		f = os.fdopen(fd, "w")
		#f = open(path, "w")
		try:
			level.write_to_file(f)
		finally:
			f.close()
		os.rename(temp_path, path)
		level.changed = False

	def get_current_playing_level_list(self):
		level_dir = self.get_current_playing_level_dir()
		pat = os.path.join(level_dir, "*%s" % Level.level_file_suffix)
		#print "Game: globbing %r" % pat ###
		return glob(pat)

	def get_current_playing_level_dir(self):
		return self.current_level_dir \
			or settings.get_default_playing_level_dir()

	def get_current_custom_level_dir(self):
		return self.current_level_dir \
			or settings.get_default_custom_level_dir()

	def get_next_level_path(self):
		paths = self.get_current_playing_level_list()
		try:
			i = paths.index(self.current_level.pathname)
		except ValueError:
			return None
		i += 1
		if i < len(paths):
			return paths[i]
