#
#   5B - Selector
#

from math import sin, cos
from pygame import draw
from part import Part
from vector_algebra import deg, Point2, Vector2

RELEASED = 0
SEIZED = 1
STEPPING = 2
SETTLING = 3
CONNECTED = 4
RELEASE_DELAY = 5
RELEASING = 6

class Selector(Part):
	#  state    int
	#  setting  int  0 to 10
	#  timeout  int

	image_name = "parts/group_selector.png"
	armature_color = (128, 128, 128)
	rotor_color = (255, 125, 13)
	terminal_points = [
		(55, 40),  # magnet 1
		(55, 15),  # magnet 2
		(55, -10), # rotor
		(-30, 55), (-40, 45), (-50, 35), (-55, 20), (-55, 5), # contacts 1-5
		(-55, -10), (-50, -25), (-40, -35), (-30, -45), (-15, -55) # contacts 6-10
	]
	armature_hinge_point = (40, -5)
	armature_end_points = [(10, -5), (10, 0)]
	rotor_center_point = (0, 5)
	rotor_length = 32
	rotor_start_angle = 90 * deg
	rotor_step_angle = 18 * deg

	def reset(self):
		self.rotor_setting = 0
		self.armature_state = 0
		self.timeout = 0
		self.state = RELEASED
	
	def draw(self, surf, t):
		tt = t * self.local_to_global_transformation()
		m = self.armature_state
		a1 = tt * Point2(self.armature_hinge_point)
		a2 = tt * Point2(self.armature_end_points[m])
		r1 = tt * Point2(self.rotor_center_point)
		ra = self.rotor_start_angle + self.rotor_setting * self.rotor_step_angle
		rr = self.rotor_length
		rv = Vector2(rr * cos(ra), rr * sin(ra))
		r2 = r1 + tt * rv
		draw.line(surf, self.armature_color, a1.coords, a2.coords, 3)
		draw.line(surf, self.rotor_color, r1.coords, r2.coords, 5)
		Part.draw(self, surf, t)
	
	def begin_frame(self):
		state = self.state
		magnet_powered = self.power_across(0, 1)
		#print "Selector: state =", self.state, "timeout =", self.timeout, \
		#	"magnet =", magnet_powered, "rotor =", self.rotor_setting ###
		#print "magnet node states:", self.node_state(0), self.node_state(1) ###
		self.armature_state = magnet_powered \
			or state in (CONNECTED, RELEASE_DELAY)
		if state == RELEASED:
			if magnet_powered:
				self.state = SEIZED
		elif state == SEIZED:
			if not magnet_powered:
				self.timeout = 4
				self.state = STEPPING
		elif state == STEPPING:
			if magnet_powered:
				if self.rotor_setting < 10:
					self.rotor_setting += 1
				self.timeout = 4
				self.state = SETTLING
			else:
				self.timeout -= 1
				if self.timeout <= 0:
					self.state = RELEASING
		elif state == SETTLING:
			if not magnet_powered:
				self.timeout = 4
				self.state = STEPPING
			self.timeout -= 1
			if self.timeout <= 0:
				self.state = CONNECTED
		elif state == CONNECTED:
			if not magnet_powered:
				self.timeout = 4
				self.state = RELEASE_DELAY
		elif state == RELEASE_DELAY:
			if magnet_powered:
				self.state = CONNECTED
			else:
				self.timeout -= 1
				if self.timeout <= 0:
					self.state = RELEASING
		elif state == RELEASING:
			if self.rotor_setting > 0:
				self.rotor_setting = max(0, self.rotor_setting - 2)
			else:
				self.state = RELEASED

	def node_changed(self, i):
		if self.state in (CONNECTED, RELEASE_DELAY):
			j = self.rotor_setting
			if j > 0:
				self.propagate(2, 2 + j)
