#
#   5B - Playing screen
#

import os
from pygame import Rect
from pygame.locals import *
from albow.widget import Widget
from albow.controls import Label, Row
from albow.resource import get_image, get_font
from screen import Screen
from level_view import LevelView
import settings
from settings import frames_to_seconds

footer_font = get_font(18, "Vera.ttf")
level_name_font = get_font(18, "VeraBd.ttf")
level_name_color = (255, 255, 255)
#level_name_width = 300
message_font = get_font(18, "VeraBd.ttf")
message_color = (255, 255, 0)
time_font = get_font(18, "VeraBd.ttf")
tyger_sniggering_image = get_image("tyger_sniggering.png")
tyger_thwarted_image = get_image("tyger_thwarted.png")
tyger_gotcha_image = get_image("tyger_gotcha.png")

#---------------------------------------------------------------------------

class LevelScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		sw, sh = settings.screen_size
		game = shell.game
		level_name_view = LevelNameDisplay(game)
		#level_name_view.rect.width = level_name_width
		#message_view = MessageDisplay(game)
		clock_view = TimeDisplay(game, 'current_time', (255, 255, 255))
		deadline_view = TimeDisplay(game, 'deadline', (255, 128, 0))
		def fl(text):
			return Label(text, font = footer_font, fg_color = (255, 255, 0))
		items = [
			fl("Level"),
			level_name_view,
			#message_view,
			fl("Time"),
			clock_view,
			fl("Deadline"),
			deadline_view,
		]
		footer = Row(items, width = sw, expand = level_name_view)
		footer.rect.bottomleft = (0, sh)
		self.level_view = PlayingView(
			Rect((0, 0), (sw, sh - footer.rect.height)), game)
		self.add(self.level_view)
		self.add(footer)
		self.level_view.focus()

	def key_down(self, e):
		key = e.key
		if key == K_ESCAPE:
			self.shell.show_menu()
		elif key == K_SPACE:
			self.space_key()
		else:
			Screen.key_down(self, e)
	
	def space_key(self):
		level = self.game.current_level
		if level.succeeded():
			self.shell.next_level()
		elif level.failed():
			self.shell.start_level()
	
	def begin_frame(self):
		level = self.game.current_level
		level.begin_frame()
		if level.auto_end():
			level.reset()
			self.shell.show_menu()

#---------------------------------------------------------------------------

class LevelNameDisplay(Widget):

	font = level_name_font

	def __init__(self, game):
		Widget.__init__(self, Rect((0, 0), self.font.size("X")))
		self.game = game
	
	def draw(self, surf):
		#self.draw_message(surf) or
		self.draw_level_name(surf)
	
#	def draw_message(self, surf):
#		mess = ""
#		level = self.game.current_level
#		if level.succeeded():
#			mess = "Press Space for Next Level"
#		elif level.failed():
#			mess = "Press Space to Restart Level"
#		if mess:
#			frame = surf.get_rect()
#			buf = self.font.render(mess, True, message_color)
#			r = buf.get_rect()
#			r.center = frame.center
#			surf.blit(buf, r)
#			return True

	def draw_level_name(self, surf):
		path = self.game.current_level.pathname
		name = os.path.basename(path)
		name = os.path.splitext(name)[0]
		name = name.replace("-", ": ")
		name = name.replace("_", " ")
		while name.startswith("0"):
			name = name[1:]
		buf = self.font.render(name, True, level_name_color)
		surf.blit(buf, (0, 0))

#---------------------------------------------------------------------------

#class MessageDisplay(Widget):
#
#	font = message_font
#	fg_color = message_color
#
#	def __init__(self, game):
#		Widget.__init__(self, Rect((0, 0), self.font.size("X")))
#		self.game = game
#	
#	def draw(self, surf):
#		mess = ""
#		level = self.game.current_level
#		if level.succeeded():
#			mess = "Press Space for Next Level"
#		elif level.failed():
#			mess = "Press Space to Restart Level"
#		if mess:
#			frame = surf.get_rect()
#			buf = self.font.render(mess, True, self.fg_color)
#			r = buf.get_rect()
#			r.center = frame.center
#			surf.blit(buf, r)

#---------------------------------------------------------------------------

def twelve_hour(hm):
	h = hm // 60
	m = hm % 60
	if h < 12:
		ap = "AM"
	else:
		h -= 12
		ap = "PM"
	if h == 0:
		h = 12
	return "%2d:%02d" % (h, m), ap

#---------------------------------------------------------------------------

class TimeDisplay(Widget):

	font = time_font
	
	def __init__(self, game, attr, color):
		w1, h1 = self.font.size("99:99")
		#w2, h2 = self.font.size(" ")
		w2 = h1 // 8
		w3, h3 = self.font.size("AM")
		r = Rect((0, 0), (w1 + w2 + w3, h1))
		Widget.__init__(self, r)
		self.game = game
		self.attr = attr
		self.fg_color = color
		self.ap_width = w3
	
	def draw(self, surf):
		time_in_frames = getattr(self.game.current_level, self.attr)
		if time_in_frames > -1:
			frame = surf.get_rect()
			time = frames_to_seconds(time_in_frames)
			text, ap = twelve_hour(time)
			buf = self.font.render(text, True, self.fg_color)
			surf.blit(buf, (0, 0))
			buf = self.font.render(ap, True, self.fg_color)
			surf.blit(buf, (frame.right - self.ap_width, 0))

#---------------------------------------------------------------------------

class PlayingView(LevelView):

	focus_finger = get_image("focus_finger.png")

	def draw_with_transform(self, surf, t, with_selection):
		LevelView.draw_with_transform(self, surf, t, with_selection)
		level = self.level
		part = level.focused_part
		if part:
			r = part.get_translated_global_rect(t)
			x, y = r.bottomright
			surf.blit(self.focus_finger, (x - 20, y - 20))
		if level.tyger_thwarted():
			self.draw_tyger(surf, tyger_thwarted_image)
		elif level.tyger_active():
			if level.tyger_succeeded():
				self.draw_tyger(surf, tyger_gotcha_image)
			else:
				self.draw_tyger(surf, tyger_sniggering_image)
	
	def draw_tyger(self, surf, image):
		frame = surf.get_rect()
		r = image.get_rect()
		r.left = frame.left + 10
		r.bottom = frame.bottom - 10
		surf.blit(image, r)

	def mouse_down(self, e):
		part = self.find_clicked_item(e, parts_only = True)
		if part and part.selectable_at_play_time():
			self.level.focused_part = part

	def key_down(self, e):
		part = self.level.focused_part
		if part and part.key_down(e):
			return
		else:
			LevelView.key_down(self, e)
	
	def draw(self, surf):
		LevelView.draw(self, surf)
		self.draw_message(surf)

	def draw_message(self, surf):
		mess = ""
		level = self.level
		if level.succeeded():
			mess = "Press Space for Next Level"
		elif level.failed():
			mess = "Press Space to Restart Level"
		if mess:
			frame = surf.get_rect()
			buf = message_font.render(mess, True, message_color)
			r = buf.get_rect()
			x, y = frame.midbottom
			r.midbottom = (x, y - 10)
			surf.blit(buf, r)
			return True
